import collections
from thingspace.env import Env
from thingspace.models.factories.contacts_factories import ContactsFactories
from thingspace.models.factories.fops_factories import FopsFactories
from thingspace.packages.requests.requests import Request
from thingspace.exceptions import CloudError, NotFoundError, ConflictError


class Contacts:

    def contacts(self, page=1, count=200, query=None, sort=None):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        params = {
            "page": page,
            "count": count,
        }
        if sort:
            params['sort'] = sort
        if query:
            params['query'] = query

        resp = self.networker(Request(
            'GET',
            Env.api_url + '/cloud/' + Env.api_version + '/contacts',
            headers=headers, params=params
        ))

        if resp.status_code != 200:
            raise CloudError("Could not get contacts", response=resp)

        json = resp.json()

        contacts = ContactsFactories.contacts_from_json(json['contactsResponse']['account']['addrbook']['contacts'].get('contact', []))
        return contacts