import collections
from thingspace.env import Env
from thingspace.models.factories.fops_factories import FopsFactories
from thingspace.packages.requests.requests import Request
from thingspace.exceptions import CloudError, NotFoundError, ConflictError


class Favorites:

    def favorites(self, virtualfolder='VZMOBILE', type=None, filetype=None):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        params = {
            "virtualfolder": virtualfolder,

        }
        if type:
            params['type'] = type
        if filetype:
            params['filetype'] = filetype

        resp = self.networker(Request(
            'GET',
            Env.api_url + '/cloud/' + Env.api_version + '/favorites',
            headers=headers, params=params
        ))


        if resp.status_code != 200:
            raise CloudError("could not get favorites", response=resp)

        FavoritesResponse = collections.namedtuple('FavoritesResponse', 'files folders')

        json = resp.json()

        files = FopsFactories.files_from_json(self, json['favorites'].get('file',[]))
        folders = FopsFactories.folders_from_json(json['favorites'].get('folder',[]))

        return FavoritesResponse(files, folders)

    def favorite(self, uris, createversion=False):
        if not self.authenticated:
            return None

        if isinstance(uris, str):
            uris = [uris]

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        body = {
            "uri": uris,
            "createversion": createversion,
        }
        resp = self.networker(Request(
            'PUT',
            Env.api_url + '/cloud/' + Env.api_version + '/favorites',
            json=body,
            headers=headers,
        ))
        if resp.status_code == 404:
            raise NotFoundError("uri not found", response=resp)

        if resp.status_code != 204:
            raise CloudError("could not add to favorites", response=resp)

        return

    def unfavorite(self, uri, createversion=False):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        params = {
            "uri": uri,
            "createversion": createversion,
        }
        resp = self.networker(Request(
            'DELETE',
            Env.api_url + '/cloud/' + Env.api_version + '/favorites',
            params=params,
            headers=headers,
        ))
        if resp.status_code == 404:
            raise NotFoundError("uri not found", response=resp)

        if resp.status_code != 204:
            raise CloudError("could not remove from favorites", response=resp)

        return