import collections
from thingspace.env import Env
from thingspace.models.factories.fops_factories import FopsFactories
from thingspace.models.factories.shares_factories import SharesFactories
from thingspace.models.share_request import ShareRequest
from thingspace.packages.requests.requests import Request
from thingspace.exceptions import CloudError, NotFoundError, ConflictError
from thingspace.packages.requests.requests.packages.urllib3.packages.six.moves import urllib


class Shares:

    def shares(self, count=20, cursor=None, filter=None, since=None, until=None):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        params = {
            "count": count,

        }
        if cursor:
            params['cursor'] = cursor
        if filter:
            params['filter'] = filter
        if since:
            params['since'] = since
        if until:
            params['until'] = until

        resp = self.networker(Request(
            'GET',
            Env.api_url + '/cloud/' + Env.api_version + '/shares',
            headers=headers, params=params
        ))


        if resp.status_code != 200:
            raise CloudError("could not get shares", response=resp)

        json = resp.json()
        shares_cursor = SharesFactories.sharecursor_from_json(json)
        shares = SharesFactories.shares_from_json(json['shares'].get('share', []))

        SharesResponse = collections.namedtuple('SharesResponse', 'shares shares_cursor')
        return SharesResponse(shares, shares_cursor)

    def share(self, resources, name=None, expires_seconds=86400):
        if not self.authenticated:
            return None

        if isinstance(resources, ShareRequest):
            resources = [resources]

        for resource in resources:
            resource.validate()

        headers = {
            "Authorization": "Bearer " + self.access_token
        }

        body = {
            "resources": resources,
        }

        if name:
            body['name'] = name
        if expires_seconds:
            body['expiresIn'] = expires_seconds

        resp = self.networker(Request(
            'POST',
            Env.api_url + '/cloud/' + Env.api_version + '/shares',
            json=body,
            headers=headers,
        ))

        # check for 400 because on some accounts our API returns 400.....
        if resp.status_code == 404 or resp.status_code == 400:
            raise NotFoundError("uri not found", response=resp)

        if resp.status_code != 201:
            raise CloudError("could not share", response=resp)

        json = resp.json()
        share = SharesFactories.share_from_json(json['share'])

        return share

    def unshare(self, uid):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        resp = self.networker(Request(
            'DELETE',
            Env.api_url + '/cloud/' + Env.api_version + '/shares/' + urllib.parse.quote(uid),
            headers=headers,
        ))
        if resp.status_code == 404:
            raise NotFoundError("share not found", response=resp)

        if resp.status_code != 204:
            raise CloudError("could not unshare", response=resp)

        return