import collections
from thingspace.env import Env
from thingspace.models.factories.fops_factories import FopsFactories
from thingspace.packages.requests.requests import Request
from thingspace.exceptions import CloudError, NotFoundError, ConflictError
from thingspace.packages.requests.requests.packages.urllib3.packages.six.moves import urllib


class Tags:

    def tags(self, path):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }

        resp = self.networker(Request(
            'GET',
            Env.api_url + '/cloud/' + Env.api_version + '/tags' + urllib.parse.quote(path),
            headers=headers,
        ))

        if resp.status_code == 404:
            raise NotFoundError("path not found", response=resp)

        # returns 200 if it has tags, 204 if no tags
        if resp.status_code != 200 and resp.status_code != 204:
            raise CloudError("could not get tags", response=resp)

        if resp.status_code == 204:
            return None


        json = resp.json()
        return json['Tags']

    def tag(self, uri, tags,  createversion=False):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        body = {
            "Tags": tags,
            "uri": uri,
            "createversion": createversion,
        }
        resp = self.networker(Request(
            'PUT',
            Env.api_url + '/cloud/' + Env.api_version + '/tags',
            json=body,
            headers=headers,
        ))
        if resp.status_code == 404:
            raise NotFoundError("uri not found", response=resp)

        if resp.status_code != 200:
            raise CloudError("could not set tags", response=resp)

        json = resp.json()
        return json['Tags']

    def untag(self, uri, createversion=False):
        if not self.authenticated:
            return None

        headers = {
            "Authorization": "Bearer " + self.access_token
        }
        params = {
            "uri": uri,
            "createversion": createversion,
        }
        resp = self.networker(Request(
            'DELETE',
            Env.api_url + '/cloud/' + Env.api_version + '/tags',
            params=params,
            headers=headers,
        ))
        if resp.status_code == 404:
            raise NotFoundError("uri not found", response=resp)

        if resp.status_code != 204:
            raise CloudError("could not remove tags", response=resp)

        return