import collections
from thingspace.env import Env
from thingspace.models.factories.fops_factories import FopsFactories
from thingspace.packages.requests.requests import Request
from thingspace.exceptions import CloudError, NotFoundError, ConflictError
from thingspace.packages.requests.requests.packages.urllib3.packages.six.moves import urllib


class VirtualFolder:

    def create_virtualfolder(self, name):
        if not self.authenticated:
            return None
        headers = {
            "Authorization": "Bearer " + self.access_token
        }

        resp = self.networker(Request(
            'POST',
            Env.api_url + '/cloud/' + Env.api_version + '/virtualfolder/' + urllib.parse.quote(name),
            headers=headers,
        ))

        if resp.status_code != 201:
            raise CloudError("could not create virtual folder", response=resp)


        json = resp.json()
        folder = FopsFactories.folder_from_json(json['folder'])
        return folder

    def delete_virtualfolder(self, name, force=False):
        if not self.authenticated:
            return None
        params={
            'name': name,
            'force': force,
        }

        headers = {
            "Authorization": "Bearer " + self.access_token
        }

        resp = self.networker(Request(
            'DELETE',
            Env.api_url + '/cloud/' + Env.api_version + '/virtualfolder',
            headers=headers, params=params
        ))

        if resp.status_code == 404:
            raise NotFoundError("Virtual folder not found")

        if resp.status_code != 204:
            raise CloudError("could not create delete folder", response=resp)

        return


    def rename_virtualfolder(self, old_name, new_name):
        if not self.authenticated:
            return None

        body = {
            'src': old_name,
            'target': new_name,
        }

        headers = {
            "Authorization": "Bearer " + self.access_token
        }

        resp = self.networker(Request(
            'PUT',
            Env.api_url + '/cloud/' + Env.api_version + '/virtualfolder',
            headers=headers, json=body
        ))

        if resp.status_code == 404 or resp.status_code == 400:
            raise NotFoundError("virtual folder not found")

        if resp.status_code != 200:
            raise CloudError("could not rename virtualfolder", response=resp)

        json = resp.json()
        folder = FopsFactories.folder_from_json(json['folder'])
        return folder
